/*
EEPROM Emulator for uPSD34xx			(c) Tim Jackson April 2007

Emulates an EEPROM of up to 256 bytes in a uPSD34xx microcontroller using two or more flash sectors 

file eeprom.h
Header for eeprom.c

Requires: upsd3400.h from ST Microelectronics
          hardware.h  modified with actual processor type etc.  See example supplied
          
Note: May produce some harmless linker warnings due to multiple absolute address allocations
if other parts of program address PSD registers or flash memory.
*/

#define MAXLEVEL 12

#include "upsd3400.h"
#include "hardware.h"
// All the pages used for EEPROM emulation must be based at address EEPROMBASE.
// The following must be changed if secondary flash is to be used or some sectors are to not be used as EEPROM
// Can use all or part of main or secondary flash, but not both.
#define EEPages uPSD_MAIN_SECTORS		
#define EEPagesize uPSD_MAIN_SECTOR_SIZE
// This must match the sector address in specified in the PLD
#define EEPROMBASE 0x8000

#pragma NOINTPROMOTE
#pragma SIGNEDCHAR
#pragma LARGE

// Call this procedure at power-up to copy the EEPROM record and Flash record into RAM buffers
// Before call, preload the RAM buffers with default data.  In event that the flash memory is clean
// or corrupt, it will be initialised using the supplied data and record sizes.
// Returns true on coldstart, false on restore
bit EERestore( void *eeprom, unsigned char eesize, void *flashrom, unsigned char flashsize);

// Ideally call this procedure in power-fail processing to write an EEPROM record in next available 
// space.  If power fail detection is not available, then call it periodically to back up changes.  
// Preferably only call it if the data has changed since the last EESave or EERestore call.
// Returns false if unable to write even after switching to a blank sector.
bit EESave( void);

// Procedure to write the flash record.  Call this whenever the flash data is changed
// Write both Flash data and EEPROM data to a new page (and then erases the old one)
// Returns false if unable to find a writable blank sector.
bit EESaveFlash( void);

// If your program alters the page select bits, call this first to check whether the page register in use
// DO NOT call from an interrupt task - it can't safely run inside EESave or EERestore.
// Returns TRUE if erase in progress (and page switching thus forbidden)
bit EECheck( void);



